'''
Author: LetMeFly
Date: 2022-01-24 12:51:09
LastEditors: LetMeFly
LastEditTime: 2022-01-26 11:23:08
'''
import requests
from . import Login  # 由账号密码到登录后的cookie
from . import GetContestProblemList  # 获取比赛有多少题目
from . import FromPidToChar  # 从题目的数字编号到字母编号
from . import GetASourceCodeAndChange  # 用Admin获取一道通过的C++代码
from . import SubmitOneCode
import time


def finish1contest(cid: str, cookie_my: requests.cookies.RequestsCookieJar, cookie_admin: requests.cookies.RequestsCookieJar, sleep_time: int) -> None:
    problem_num = GetContestProblemList.getContestProblemList(cid, cookie_my)
    print(f"比赛{cid}共有{problem_num}道题目")
    for pid in range(problem_num):
        pid_str = FromPidToChar.i2s(pid)
        try:
        # if True:
            code = GetASourceCodeAndChange.getASourceCodeAndChange(cid, pid_str, cookie_admin)
        except:
        # else:
            print(f"Problem{pid_str}暂未有C++通过者")
            continue
        SubmitOneCode.submit(cid, pid, cookie_my, code)
        print(f"Problem{pid_str}已提交")
        time.sleep(sleep_time)
        

def main(cid: str, username_my, password_my, username_admin, password_admin, sleep_time = 15) -> None:
    """
    自动Copy代码并完成比赛 + 不被查重
        Input: 比赛的cid、要自动提交的账号的用户名和密码、Admin的用户名和密码
        Output: 完成一场比赛(前提是每道题都有人提交了C++代码)
    """
    cookie_my = Login.login(username_my, password_my)
    cookie_admin = Login.login(username_admin, password_admin)
    finish1contest(cid, cookie_my, cookie_admin, sleep_time=sleep_time)
    




'''
Author: LetMeFly
Date: 2022-01-25 18:07:26
LastEditors: LetMeFly
LastEditTime: 2022-01-26 11:05:42
'''
# __all__ = ['AutoGetAndSubmit', 'FromPidToChar', 'GetASourceCodeAndChange', 'GetContestProblemList', 'Login', 'SubmitOneCode']
from .Login import login
from .AutoGetAndSubmit import main as finish1contest
from .SubmitOneCode import submit



'''
Author: LetMeFly
Date: 2022-01-24 15:18:26
LastEditors: LetMeFly
LastEditTime: 2022-01-26 11:00:24
Description:
    从题目编号(数字)到题目编号(字母)
        0: A
    1: B
    2: C
    ...
    25: Z
    26: AA
    27: AB
    ...
    51: AZ
    52: BA
    ...
    701: ZZ
    702: ''
    703: ''
    
    只有前701道题有编号，也就是说字母编号只有两位
    低位从A~Z，相当于26进制
    高位从''(空)到A~Z，相当于27进制
    表达范围：[0, 27 * 26) = [0, 701]
'''
def i2s(num: int) -> str:
    if num < 26:
        return chr(ord('A') + num)
    else:
        return chr(ord('A') + num // 26 - 1) + chr(ord('A') + num % 26)



'''
Author: LetMeFly
Date: 2022-01-24 16:04:55
LastEditors: LetMeFly
LastEditTime: 2022-01-24 17:01:41
'''
import requests
from bs4 import BeautifulSoup
import random
import string
try:
    import lxml
    features = 'lxml'
except:
    featrues = None

import requests
from bs4 import BeautifulSoup
try:
    import lxml
    features = 'lxml'
except:
    featrues = None


def getASourceCode(cid: str, pid: str, cookies: requests.cookies.RequestsCookieJar) -> str:
    """
    通过admin的cookie获得一个通过的C++代码(前提是有人通过)
    """
    url = f"http://182.92.175.181/status.php?cid={cid}&problem_id={pid}&user_id=&language=1&jresult=4&showsim=0"
    response = requests.get(url, cookies=cookies)
    soup = BeautifulSoup(response.text, features)
    table = soup.find('table', attrs={"id": "vueAppFuckSafari"})
    tr = table.find('tbody').find('tr')
    td = tr.find_all('td')[6]
    a = td.find_all('a')[1]
    href = 'http://182.92.175.181/' + a.get('href')
    response = requests.get(href, cookies=cookies)
    soup = BeautifulSoup(response.text, features)
    code = soup.find('pre').string
    return code


def randText() -> str:
    return "".join(random.choice(string.ascii_letters + string.digits + "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n{()}-+*-.><") for _ in range(2048))


def getASourceCodeAndChange(cid: str, pid: str, cookies: requests.cookies.RequestsCookieJar) -> str:
    code = getASourceCode(cid=cid, pid=pid, cookies=cookies)
    pre = f"""// LetMeFly_PySubmiter
#include <bits/stdc++.h>
using namespace std;
#define mem(a) memset(a, 0, sizeof(a))
#define dbg(x) cout << #x << " = " << x << endl
#define fi(i, l, r) for (int i = l; i < r; i++)
#define cd(a) scanf("%d", &a)
typedef long long ll;
#ifdef LetMeFly_PySubmiter
{randText()}
#endif
"""
    return pre + code
